# Buddha Machine soundscape simulator
# Code released into the public domain
# by Phil Bordelon <phil thenexusproject org>
# and Andrew Plotkin <erkyrath eblong com>
#
# Download the Buddha Machine loops at:
#
# http://www.fm3buddhamachine.com/

import boodle.agent
import play
import random

sounddir = 'buddha loops loud'

buddha_loops = [
	'buddha machine01.wav',
	'buddha machine02.wav',
	'buddha machine03.wav',
	'buddha machine04.wav',
	'buddha machine05.wav',
	'buddha machine06.wav',
	'buddha machine07.wav',
	'buddha machine08.wav',
	'buddha machine09.wav',
]
buddha_loops = [ sounddir+'/'+val for val in buddha_loops ]

Agent = boodle.agent.Agent

class BuddhaLoop (Agent):
	name = 'Specific Buddha Machine loop'

	def __init__ (self, loop_num=1, pitch=1):
		Agent.__init__ (self)
		choice = int(loop_num)
		self.pitch = float(pitch)
		if choice < 1 or choice > len (buddha_loops):
			raise ValueError("Choice must be between 1 and %d" % len(buddha_loops))
		else:
			self.sound = buddha_loops[choice - 1]

	def run (self):
		duration = self.sched_note (self.sound, self.pitch)
		self.resched (duration)

class RandomLoop (Agent):
	name = 'Random Buddha Machine loop'

	def __init__ (self, pitch=1):
		Agent.__init__ (self)
		self.pitch = float(pitch)

	def run (self):
		val = random.randint(1, len(buddha_loops))
		self.sched_agent(BuddhaLoop(val, self.pitch))

class ChangingLoops (Agent):
	name = 'Shift slowly from one loop to another'

	def __init__(self, mindelay=60, maxdelay=180, fadetime=6, pitch=1,
		interval=0):
		Agent.__init__(self)
		self.mindelay = float(mindelay)
		self.maxdelay = float(maxdelay)
		self.pitch = float(pitch)
		self.fadetime = float(fadetime)
		self.interval = float(interval)

	def run (self):
		ag = RandomLoop(self.pitch)
		dur = random.uniform(self.mindelay, self.maxdelay)
		fader = play.FadeInOutAgent(ag, dur, self.fadetime)
		self.sched_agent(fader)
		self.resched(dur + self.fadetime + self.interval)
		
class ChangingLoopsPitches (Agent):
	name = 'Shift slowly from one loop to another, varying key'

	pitchlist = [
		0.666666, 0.75, 0.8, 0.833333, 1.0, 1.2, 1.25, 1.333333, 1.5,
	]

	def __init__(self, mindelay=60, maxdelay=180, fadetime=6, interval=0):
		Agent.__init__(self)
		self.mindelay = float(mindelay)
		self.maxdelay = float(maxdelay)
		self.fadetime = float(fadetime)
		self.interval = float(interval)

	def run (self):
		pitch = random.choice(self.pitchlist)
		ag = RandomLoop(pitch)
		dur = random.uniform(self.mindelay, self.maxdelay)
		fader = play.FadeInOutAgent(ag, dur, self.fadetime)
		self.sched_agent(fader)
		self.resched(dur + self.fadetime + self.interval)
		
class Layers (Agent):
	name = 'Two layers of Buddha Machine'

	def run (self):
		self.sched_agent(ChangingLoops())
		self.sched_agent(ChangingLoops(30, 90, 30, 0.8, 30))

class ComplexLayers (Agent):
	name = 'Multiple layers of Buddha Machine with changing pitches'

	def __init__(self, layers=2, mindelay=60, maxdelay=180, fadetime=6):
		Agent.__init__(self)

		self.layers = int(layers)
		self.mindelay = float(mindelay)
		self.maxdelay = float(maxdelay)
		self.fadetime = float(fadetime)

		if self.layers < 1:
			raise ValueError ("Must have at least one layer")

	def run (self):
		for ix in range(self.layers):
			ag = ChangingLoopsPitches(self.mindelay, self.maxdelay,
				self.fadetime, 10*ix)
			self.sched_agent(ag)

